@@ -23,11 +23,20 @@
 #include <fstream>
 #include <iostream>
 
+// For video4linux access
+#include <sys/ioctl.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <linux/videodev.h>
+#include <linux/videodev2.h>
+
 #include <gst/interfaces/colorbalance.h>
 #include <gst/interfaces/xoverlay.h>
 #include <gst/interfaces/videoorientation.h>
 #include <gst/video/video.h>
 
+#include <Lum/Base/String.h>
 #include <Lum/Base/Util.h>
 
 #include <Lum/Dialog.h>
@@ -39,6 +48,14 @@
 
 #define CAMERA_ORIENTATION_FILE "/sys/devices/platform/gpio-switch/cam_turn/state"
 
+class Device
+{
+public:
+  std::string name;
+  std::string device;
+  std::string gstSink;
+};
+
 static gboolean cb_have_data(GstElement* /*element*/,
                              GstBuffer *buffer,
                              GstPad* /*pad*/,
@@ -56,6 +73,101 @@ gboolean GstCallback(GstBus* bus, GstMessage* message, gpointer user_data)
   return true;
 }
 
+void GetV4LDevices(std::list<Device>& devices)
+{
+  std::string devicePrefix="/dev/video";
+  size_t      deviceIndex=0;
+
+  devices.clear();
+
+  while (true) {
+    std::string             deviceName=devicePrefix+Lum::Base::NumberToString(deviceIndex);
+    int                     handle;
+    struct video_capability caps1;
+    struct v4l2_capability  caps2;
+
+    handle=open(deviceName.c_str(),O_RDWR);
+
+    if (handle==-1) {
+      return;
+    }
+
+    std::cout << "Detected device " << deviceName << std::endl;
+
+    std::cout << "  Checking V4L2 compatibility..." << std::endl;
+
+    if (ioctl(handle,VIDIOC_QUERYCAP,&caps2)!=-1) {
+      std::cout << "    Got capabilities...\"" << caps2.driver << "\" \"" << caps2.card << "\" \"" << caps2.bus_info << "\"" << std::endl;
+
+      if (caps2.capabilities & V4L2_CAP_VIDEO_CAPTURE) {
+        std::cout << "    Video source can be captured..." << std::endl;
+
+        size_t index=0;
+
+        while (true) {
+          struct v4l2_input input;
+
+          input.index=index;
+
+          if (ioctl(handle,VIDIOC_ENUMINPUT,&input)!=-1) {
+            if (input.type & V4L2_INPUT_TYPE_CAMERA) {
+              std::cout << "    Input " << index << " is of type camera!" << std::endl;
+
+              Device device;
+
+              device.name=(const char*)caps2.card;
+              device.device=deviceName;
+              device.gstSink="v4l2src";
+
+              devices.push_back(device);
+            }
+
+            index++;
+          }
+          else {
+            break;
+          }
+        }
+      }
+    }
+
+    std::cout << "  Checking V4L compatibility..." << std::endl;
+
+    if (ioctl(handle,VIDIOCGCAP,&caps1)!=-1) {
+      std::cout << "    Got capabilities...\"" << caps1.name << "\"" << std::endl;
+
+      if (caps1.type & VID_TYPE_CAPTURE) {
+        std::cout << "    Video source can be captured..." << std::endl;
+        for (size_t c=0; c<caps1.channels; c++) {
+          struct video_channel channel;
+
+          channel.channel=c;
+
+          if (ioctl(handle,VIDIOCGCHAN,&channel)!=-1) {
+            if (channel.type & VIDEO_TYPE_CAMERA) {
+              std::cout << "    Channel " << c << " is of type camera!" << std::endl;
+
+              Device device;
+
+              device.name=caps1.name;
+              device.device=deviceName;
+              device.gstSink="v4lsrc";
+
+              devices.push_back(device);
+            }
+          }
+        }
+      }
+    }
+
+    close(handle);
+
+    std::cout << "done." << std::endl;
+
+    deviceIndex++;
+  }
+}
+
 VideoControl::VideoControl()
 : pipeline(NULL),videoSink(NULL),imageSink(NULL),
   source(NULL),
@@ -180,39 +292,48 @@ void VideoControl::CalcSize()
 void VideoControl::Layout()
 {
   if (!done) {
-    GError *error=NULL;
+    GError            *error=NULL;
+    std::list<Device> devices;
+    std::string       videoSinkName;
+    std::string       launchString;
 
     gst_init(NULL,NULL);
 
-    // Pipeline elements
-    /*
-    pipeline=::gst_parse_launch("videotestsrc name=videosource ! "
-                                "video/x-raw-yuv ! "
-                                "ffmpegcolorspace ! "
-                                "tee name=t ! "
-                                "queue ! "
-                                "{ xvimagesink name=videosink }"
-                                "t. ! queue ! "
-                                "{ ffmpegcolorspace ! "
-                                "video/x-raw-rgb,bpp=24,depth=24,framerate:fraction=8/1 !"
-                                "fakesink name=imagesink }",
-                                &error);*/
+    GetV4LDevices(devices);
 
-    pipeline=::gst_parse_launch("v4l2src name=videosource ! "
-                                "video/x-raw-yuv,width=640,height=480 ! "
-                                "ffmpegcolorspace ! "
-                                "tee name=t ! "
-                                "queue ! "
-                                "{ xvimagesink name=videosink }"
-                                "t. ! "
-                                "queue ! "
-                                "{ ffmpegcolorspace ! "
-                                "video/x-raw-rgb,bpp=24,depth=24,framerate=(fraction)8/1 ! "
-                                "fakesink name=imagesink }",
+    if (devices.size()>0) {
+      videoSinkName=devices.front().gstSink+" device="+devices.front().device;
+    }
+    else {
+      videoSinkName="videotestsrc";
+    }
+
+    launchString="v4l2src device=/dev/video0 name=videosource ! ffmpegcolorspace ! xvimagesink name=videosink";
+
+/*
+    launchString=videoSinkName+" "+
+                 "name=videosource ! "
+                 "ffmpegcolorspace ! "
+                 "video/x-raw-rgb,width=640,height=480 ! "
+                 "tee name=tee tee. ! "
+                 "queue ! "
+                 "{ xvimagesink name=videosink } "
+                 "tee. ! "
+                 "queue ! "
+                 "{ ffmpegcolorspace ! "
+                 "video/x-raw-rgb,bpp=24,depth=24,framerate=(fraction)15/1 ! "
+                 "fakesink name=imagesink }";*/
+
+    //"video/x-raw-rgb,bpp=24,depth=24,frameRate=(fraction)15/1 ! "
+    std::cout << "Launch string is: \""+launchString+"\"" << std::endl;
+
+    // Pipeline elements
+
+    pipeline=::gst_parse_launch(launchString.c_str(),
                                 &error);
 
     if (pipeline==NULL) {
-      std::cerr << "Cannot open video source" << std::endl;
+      std::cerr << "Cannot open video source: " << error << std::endl;
       done=true;
       return;
     }
@@ -221,10 +342,15 @@ void VideoControl::Layout()
     videoSink=gst_bin_get_by_name(GST_BIN(pipeline),"videosink");
     imageSink=gst_bin_get_by_name(GST_BIN(pipeline),"imagesink");
 
-    assert(videoSource!=NULL && videoSink!=NULL && imageSink!=NULL);
+    assert(videoSource!=NULL);
+    assert(videoSink!=NULL);
 
-    g_object_set(G_OBJECT(imageSink),"signal-handoffs",TRUE,NULL);
-    g_signal_connect(imageSink,"handoff",G_CALLBACK(cb_have_data),this);
+    if (imageSink!=NULL) {
+      assert(imageSink!=NULL);
+
+      g_object_set(G_OBJECT(imageSink),"signal-handoffs",TRUE,NULL);
+      g_signal_connect(imageSink,"handoff",G_CALLBACK(cb_have_data),this);
+    }
 
     Lum::OS::GMainLoopEventLoop* eventLoop=dynamic_cast<Lum::OS::GMainLoopEventLoop*>(Lum::OS::display->GetEventLoop());
     GstBus*                      bus=gst_element_get_bus(pipeline);
@@ -246,12 +372,12 @@ void VideoControl::Layout()
       if (gst_video_get_size(videoSourcePad,&w,&h)) {
         videoWidth=w;
         videoHeight=h;
+        std::cout << "Image dimension: " << videoWidth << "x" << videoHeight << std::endl;
       }
 
       gst_object_unref(videoSourcePad);
     }
 
-
     Lum::OS::display->AddTimer(1,0,timer);
 
     done=true;
@@ -316,6 +442,34 @@ void VideoControl::OnGstMessage(GstMessage* message)
   }
 }
 
+bool VideoControl::HandleMouseEvent(const Lum::OS::MouseEvent& event)
+{
+  if (!knips->IsEnabled()) {
+    return false;
+  }
+
+  if (event.type==Lum::OS::MouseEvent::down &&
+      PointIsIn(event) &&
+      event.button==Lum::OS::MouseEvent::button1) {
+      if (!knips->IsStarted()) {
+        knips->Start();
+        return true;
+      }
+    }
+    else if (event.IsGrabEnd()) {
+      if (knips->IsStarted()) {
+        if (PointIsIn(event)) {
+            knips->Finish();
+        }
+        else {
+          knips->Cancel();
+        }
+      }
+    }
+
+  return false;
+}
+
 void VideoControl::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
 {
   if (model==timer && timer->IsFinished()) {
@@ -327,6 +481,7 @@ void VideoControl::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& m
            model==dynamic_cast<Lum::Dialog*>(GetWindow()->GetMaster())->GetOpenedAction() &&
            dynamic_cast<Lum::Dialog*>(GetWindow()->GetMaster())->GetOpenedAction()->IsFinished() &&
            videoSink!=NULL) {
+    std::cout << "Assigning window to overlay..." << std::endl;
     gst_x_overlay_set_xwindow_id(GST_X_OVERLAY(videoSink),window);
   }
 
